import cnvrg.helpers.apis_helper as apis_helper
import cnvrg.helpers.param_build_helper as param_build_helper

class BaseConnector():
    def __init__(self, data_connector, info=None):
        org, data_connector = param_build_helper.parse_params(data_connector, param_build_helper.DATA_CONNECTOR)
        self._org = org
        self._data_connector = data_connector
        self.info = None
        self.__init_data_connector(info=info)

    def connector_type(self):
        return self.info.get("connector_type")

    def __init_data_connector(self, info=None):
        self.info = info or apis_helper.get(self.__base_url()).get("data_connector")
        self.data = self.info.get("credentials")

    def __base_url(self):
        return apis_helper.url_join('users', self._org, "data_connectors", self._data_connector)


    def torch_ds(self):
        from torch.utils.data.dataset import Dataset as TorchDataset
        class TorchDs(TorchDataset):
            def __init__(self, dc):
                self.__dc = dc
            def __getitem__(self, item):
                return self.__dc[item]
            def __len__(self):
                return len(self.__dc)
        return TorchDs(self)

    def data_loader(self, **kwargs):
        from torch.utils import data
        class DataLoader(data.DataLoader):
            def __init__(self, dc, **kwargs):
                super(DataLoader, self).__init__(dc.torch_ds(), **kwargs)
        return DataLoader(self, **kwargs)


from cnvrg.modules.data_connector.base_connector import BaseConnector
from multiprocessing import Process
class BaseStreamConnector(BaseConnector):
    def __init__(self, *args, **kwargs):
        ## topic => [callbacks]
        self.subs = {}
        ## topic => Proccess
        self.proccess = {}
        super(BaseStreamConnector, self).__init__(*args, **kwargs)

    def producer(self):
        raise Exception("Not Implemented")

    def consumer(self, *topics, **kwargs):
        raise Exception("Not Implemented")


    def __init_sub(self, topic, **kwargs):
        def __subscription(stream_connector, topic, **kwargs):
            _consumer = stream_connector.consumer(topic, **kwargs)
            for msg in _consumer:
                for cb in stream_connector.subs.get(topic):
                    cb(msg)

        t = Process(target=__subscription, args=(self, topic), kwargs=kwargs)
        t.start()
        self.proccess[topic] = t

    def subscribe(self, topic, callback, **kwargs):
        if topic not in self.subs:
            self.__init_sub(topic, **kwargs)
            self.subs[topic] = []
        self.subs[topic].append(callback)


    def unsubscribe(self, topic):
        if topic not in self.proccess:
            return
        self.proccess[topic].terminate()
        self.subs[topic] = None

    def send(self, topic, **kwargs):
        """
        Send a specific **kwargs to a topic
        :param topic: topic to send to
        :return: Future
        """
        return self.producer().send(topic, **kwargs)



from cnvrg.modules.data_connector.base_connector import BaseConnector
from cnvrg.modules.dataset import Dataset
from cnvrg.helpers.apis_helper import url_join

class DatasetConnector(BaseConnector):

    @staticmethod
    def key_type():
        return "bucket"

    def __init__(self, dataset=None, working_dir=None):
        super(DatasetConnector, self).__init__(dataset)
        self.__files = None
        self.__dataset = Dataset(url_join(self._org, self._data_connector), working_dir=working_dir)

    #
    # def get_base_url(self):
    #     return "users/{owner}/datasets/{dataset}".format(owner=self.__owner, dataset=self.__dataset)
    #
    # def fetch_all_files(self):
    #     return self.get_commit_files()

    def __len__(self):
        if not self.__files: self.__files = self.__dataset.fetch_all_files()
        return len(self.__files)

    def __getitem__(self, item):
        return self.__dataset.download_file(self.__files[item])